/*
 ** Module   : Quoter.java
 ** Abstract : Java implementation of the Quoter utility
 **
 ** Copyright (c) 2012, Golden Code Development Corporation.
 ** ALL RIGHTS RESERVED. Use is subject to license terms.
 **
 **           Golden Code Development Corporation
 **                      CONFIDENTIAL
 **
 ** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
 ** 001 CS 20121102 NEW    Created initial version.

 */
package com.goldencode.p2j.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
/**
 * A class implementation of the Progress QUOTER utility
 * <p>
 * This class is used to parse the quoter utility parameters and execute the
 * equivalent logic.
 * 
 * @author cs
 */
public class Quoter
{
   private static void
         writeLine(String[] tokens, BufferedWriter writer) throws IOException
   {
      for (int i = 0; i < tokens.length; i++)
      {
         if (i == 0)
         {
            writer.append('"');
            writer.write(tokens[i]);
            writer.append('"');
         } else if (i < tokens.length)
         {
            writer.write(" ");
            writer.append('"');
            writer.write(tokens[i]);
            writer.append('"');
         }
      }
   }
   /**
    * The entry point method
    * 
    * @param strings
    *           The parameters given to the quoter utility
    */
   public static void launch(String[] strings)
   {
      String inFile = null;
      String outFile = null;
      String delimiter = null;
      String colRanges = null;
      /*
       * (first parameter will be the utility name -> "quoter" and can be
       * ignored)
       */
      for (int i = 1; i < strings.length; i++)
      {
         if (strings[i].toUpperCase().equals("-D"))
         {
            delimiter = strings[i + 1];
         }
         if (strings[i].toUpperCase().equals("-C"))
         {
            colRanges = strings[i + 1];
         }
         if (strings[i].startsWith(">"))
         {
            inFile = strings[i - 1];
            outFile = strings[i].substring(1, strings[i].length());
         }

      }
      quoter(inFile, outFile, delimiter, colRanges);

   }
   /**
    * Main logic method
    * 
    * @param inFile
    *           The input file name
    * @param outFile
    *           The output file name
    * @param delimiter
    *           The delimiter if any
    * @param colRange
    *           The column ranges to be parsed
    */
   private static void quoter(String inFile,
                              String outFile,
                              String delimiter,
                              String colRange)
   {
      File input = new File(inFile);
      File output = new File(outFile);
      String separator = null;
      List<ColumnRange> columnRanges = null;
      if (delimiter != null)
      {
         separator = delimiter;
      }
      BufferedReader readFile = null;
      BufferedWriter destFile = null;
      try
      {
         destFile = new BufferedWriter(new FileWriter(output));
      } catch (IOException e1)
      {
         return;
      }
      if (delimiter != null && colRange != null)
      {
         try
         {
            destFile.write("Incompatible options: -c and -d ");
            destFile.close();

         } catch (IOException e)
         {
            return;
         }
         return;
      }
      if (colRange != null)
      {
         columnRanges = new ArrayList<ColumnRange>();
         String[] ranges = colRange.split(",");
         for (int i = 0; i < ranges.length; i++)
         {
            String[] columns = ranges[i].split("-");
            if (columns.length == 2)
            {
               int startCol = Integer.parseInt(columns[0]);
               int endCol = Integer.parseInt(columns[1]);
               if (startCol > endCol)
               {
                  if (destFile != null)
                  {
                     try
                     {
                        destFile.write("Invalid range: " + ranges[i]);
                        destFile.close();
                        return;
                     } catch (IOException e)
                     {
                        return;
                     }
                  }

               } else
               {
                  columnRanges.add(new ColumnRange(startCol, endCol));
               }
            }
         }
      }

      if (input.exists())
      {
         try
         {
            readFile = new BufferedReader(new FileReader(input));

         } catch (FileNotFoundException e)
         {
            /*
             * nothing special to do here this is handled by checking is
             * readFile is null
             */
         }
         if (readFile != null)
         {
            if (delimiter != null)
            {
               String line = null;
               boolean firstElement = true;
               try
               {
                  while ((line = readFile.readLine()) != null)
                  {
                     String[] tokens = line.split(separator);
                     if (firstElement)
                     {
                        writeLine(tokens, destFile);
                        firstElement = false;
                     } else
                     {
                        destFile.newLine();
                        writeLine(tokens, destFile);
                     }

                  }
               } catch (IOException e)
               {
                  try
                  {
                     destFile.close();
                  } catch (IOException e1)
                  {
                     return;
                  }
                  return;
               }
            } else
            {
               if (colRange != null)
               {
                  String line = null;
                  boolean firstElement = true;
                  try
                  {
                     while ((line = readFile.readLine()) != null)
                     {
                        if (columnRanges != null && columnRanges.size() > 0)
                        {
                           if (firstElement)
                           {
                              writeColumns(line, columnRanges, destFile);
                              firstElement = false;
                           } else
                           {
                              destFile.newLine();
                              writeColumns(line, columnRanges, destFile);
                           }
                        }

                     }
                  } catch (IOException e)
                  {
                     try
                     {
                        destFile.close();
                     } catch (IOException e1)
                     {
                        return;
                     }
                     return;
                  }

               } else
               {
                  String line = null;
                  boolean firstElement = true;
                  try
                  {
                     while ((line = readFile.readLine()) != null)
                     {
                        if (firstElement)
                        {
                           destFile.write('"' + line + '"');
                           firstElement = false;
                        } else
                        {
                           destFile.newLine();
                           destFile.write('"' + line + '"');
                        }
                     }
                  } catch (IOException e)
                  {
                     try
                     {
                        destFile.close();
                     } catch (IOException e1)
                     {
                        return;
                     }
                     return;
                  }
               }

            }

         }
      } else
      {
         if (destFile != null)
         {
            try
            {
               destFile.write("Unable to read: " + inFile);
               destFile.close();
            } catch (IOException e)
            {
               return;
            }
         }

      }
      try
      {
         if (readFile != null)
         {
            readFile.close();
         }
         destFile.close();
      } catch (IOException e)
      {

      }
   }
   
   private static void writeColumns(String line,
                                    List<ColumnRange> columnRanges,
                                    BufferedWriter writer) throws IOException
   {
      boolean first = true;
      for (ColumnRange column : columnRanges)
      {
         String columnText = null;
         if (line.length() >= column.getEndColumn())
         {
            columnText = line.substring(column.getStartColumn(),
                                        column.getEndColumn());
         } else
         {
            if (column.getStartColumn() > line.length())
            {
               columnText = null;
            } else
            {
               columnText = line.substring(column.getStartColumn());
            }
         }
         if (first)
         {
            if (columnText != null)
            {
               writer.append('"');
               writer.write(columnText);
               writer.append('"');
            } else
            {
               /* when column text is null Progres puts a "-" mark */
               writer.write("-");
            }
            first = false;
         } else
         {
            writer.write(" ");
            if (columnText != null)
            {
               writer.append('"');
               writer.write(columnText);
               writer.append('"');
            } else
            {
               /* when column text is null Progres puts a "-" mark*/
               writer.write("-");
            }
         }
      }

   }
}
/*
 * utility class to handle column ranges
 */
class ColumnRange
{
   private int startColumn;
   private int endColumn;
   public ColumnRange(int startColumn, int endColumn)
   {
      /* Java starts from 0 not from 1 */
      this.startColumn = startColumn - 1;
      /* in Progress endColumn is inclusive so no need to substract */
      this.endColumn = endColumn;
   }
   public int getStartColumn()
   {
      return startColumn;
   }
   public int getEndColumn()
   {
      return endColumn;
   }
}
